/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var PlateEvents = function()
{
	var plateevents = {};
	
	/**
		Initialize the page.
	*/
	plateevents.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			Buttons.addClickHandler('btnSave', plateevents.save);
			Buttons.addClickHandler('close', App.closeWindow);
			
			// Tab validation
			TabControl.addTabValidator('settings.info', plateevents.validateBioPlateEvent);

			// Hardware
			Buttons.addClickHandler('hardware_id.select', plateevents.selectHardware);
			Events.addEventHandler('hardware_id', 'base-selected', plateevents.setHardwareCallback);

			// Protocol
			Buttons.addClickHandler('protocol_id.select', plateevents.selectProtocol);
			Events.addEventHandler('protocol_id', 'base-selected', plateevents.setProtocolCallback);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'BIOPLATEEVENT', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'BIOPLATEEVENT'};
			var tableAttributes = {'table-id': 'events'};
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
			
			TabControl.setSwitchFunction('main', plateevents.switchTab);
		}
	}
	
	plateevents.switchTab = function(event)
	{
		var target = event.target;
		var tabId = target.id.substring(5);

		var frm = document.forms['events'];
		var bioPlateId = frm.bioplate_id.value;
		
		var url;
		if (tabId == 'biowells')
		{
			url = '../wells/index.jsp?ID='+App.getSessionId();
			url += '&bioplate_id='+bioPlateId;
		}
		else
		{
			url = '../index.jsp?ID='+App.getSessionId();
			url += '&cmd=ViewItem&item_id='+bioPlateId;
			url += '&tab='+tabId;
		}
		location.replace(url);
	}
	

	plateevents.validateBioPlateEvent = function()
	{
		var frm = document.forms['event'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		return true;
	}

	// Submit the form
	plateevents.save = function()
	{
		var frm = document.forms['event'];
		if (TabControl.validateActiveTab('settings'))
		{
			frm.submit();
		}
	}

	plateevents.selectHardware = function()
	{
		var frm = document.forms['event'];
		url = '&resetTemporary=1';
		if (frm.hardware_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.hardware_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('HARDWARE', 'hardware_id', 0, url);
	}
	
	plateevents.setHardwareCallback = function(event)
	{
		var frm = document.forms['event'];
		var list = frm.hardware_id;
		if (list.length < 2 || list[1].value == '0')
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}
	
	plateevents.selectProtocol = function()
	{
		var frm = document.forms['event'];
		url = '&resetTemporary=1';
		if (frm.protocol_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.protocol_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('PROTOCOL', 'protocol_id', 0, url);
	}
	
	plateevents.setProtocolCallback = function(event)
	{
		var frm = document.forms['event'];
		var list = frm.protocol_id;
		if (list.length < 2 || list[1].value == '0')
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}


	return plateevents;
}();

Doc.onLoad(PlateEvents.initPage);
