/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var ParseFile = function()
{
	var parse = {};
	var linePatterns;
	var columnHeaders;
	var fuzzyMatches;

	/**
		Initialize the page.
	*/
	parse.initPage = function()
	{
		linePatterns = Data.json('page-data', 'line-patterns');
		columnHeaders = Data.json('page-data', 'column-headers');
		fuzzyMatches = Data.json('page-data', 'fuzzy-matches');
		
		Buttons.addClickHandler('btnFuzzyMatches', parse.setFuzzyMatches);
		Events.addEventHandler('similarityScore', 'keypress', Events.numberOnly);
	}

	parse.initColumnMapping = function(element, autoInit)
	{
		if (autoInit == 'line-pattern')
		{
			Events.addEventHandler(element, 'change', parse.setLinePattern)
		}
		else if (autoInit == 'column-mapping-clear')
		{
			Events.addEventHandler(element, 'click', parse.clearColumnMapping);
		}
		else if (autoInit == 'column-mapping-preset')
		{
			Events.addEventHandler(element, 'change', parse.setPresetColumnMapping);
		}
		else if (autoInit == 'column-mapping')
		{
			Events.addEventHandler(element, 'blur', parse.columnMappingOnBlur);
		}
	}
	Doc.addElementInitializer(parse.initColumnMapping);
	
	parse.setLinePattern = function(event)
	{
		var target = event.currentTarget;
		var lineNo = Data.int(target, 'line-no');
		
		var whichPattern = target.value;
		target.selectedIndex = 0;
		var frm = window.parent.document.forms['testwithfile'];
		
		if (frm[whichPattern])
		{
			frm[whichPattern].value = linePatterns[lineNo];
		}
	}
	
	parse.columnMappingOnBlur = function(event)
	{
		var target = event.currentTarget;
		window.parent.TestWithFile.setFormValue(target.name, target.value);
	}
	
	parse.setColumnMapping = function(mappingName, expression)
	{
		var frm = document.forms['mappings'];
		var field = 'mapping.'+mappingName+'.expression';
		frm[field].value = expression;
		window.parent.TestWithFile.setFormValue(field, expression);
	}
	
	parse.getMappingExpressionForHeader = function(columnIndex)
	{
		var frm = document.forms['mappings'];
		var style = Forms.getCheckedRadio(frm.expressionStyle);
		var header = columnHeaders[columnIndex];

		var expression = '';
		if (style && style.value == 1)
		{
			// 'simple' style with backslashes
			// use index of the header already has a backslash
			if (header.indexOf('\\') >= 0) 
			{
				expression = '\\'+columnIndex+'\\';
			}
			else
			{
				expression = '\\'+header+'\\';
			}
		}
		else
		{
			// use 'expression' style
			if (header.indexOf('\\') >= 0)
			{
				expression = 'col('+columnIndex+')';
			}
			else
			{
				expression = 'col(\''+header+'\')';
			}
		}
		return expression;
	}
	
	parse.clearColumnMapping = function(event)
	{
		var target = event.currentTarget;
		var mappingName = Data.get(target, 'mapping');
		parse.setColumnMapping(mappingName, '');
	}
	
	parse.setPresetColumnMapping = function(event)
	{
		var target = event.currentTarget;
		var mappingName = Data.get(target, 'mapping');
	
		var columnIndex = target.value;
		target.selectedIndex = 0;
		var expression = parse.getMappingExpressionForHeader(columnIndex);

		parse.setColumnMapping(mappingName, expression);
	}
	
	parse.setFuzzyMatches = function()
	{
		var frm = document.forms['mappings'];
		var minScore = parseFloat(frm.similarity.value);
		for (var i = 0; i < fuzzyMatches.length; i++)
		{
			var match = fuzzyMatches[i];
			if (match)
			{
				var expression;
				if (match.score >= minScore)
				{
					expression = parse.getMappingExpressionForHeader(match.columnIndex);
				}
				else
				{
					expression = '';
				}
				parse.setColumnMapping(match.name, expression);
			}
		}
	}
	
	return parse;
}();

Doc.onLoad(ParseFile.initPage);