/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var BioAssays = function()
{
	var bioassays = {};
	
	/**
		Initialize the page.
	*/
	bioassays.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', bioassays.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabActivateListener('settings.annotations', bioassays.loadAnnotationsFrame);
			TabControl.addTabActivateListener('settings.datafiles', bioassays.loadDataFilesFrame);
			TabControl.addTabValidator('settings.info', bioassays.validateBioAssay);

			// Subtype
			Events.addEventHandler('subtype_id', 'change', bioassays.subtypeOnChange);

			// Extract
			Buttons.addClickHandler('extract_id.select', bioassays.selectExtract);
			Events.addEventHandler('extract_id', 'base-selected', bioassays.setExtractCallback);
			
			// Protocol
			Buttons.addClickHandler('protocol_id.select', bioassays.selectProtocol);
			Events.addEventHandler('protocol_id', 'base-selected', bioassays.setProtocolCallback);
			
			// Hardware
			Buttons.addClickHandler('hardware_id.select', bioassays.selectHardware);
			Events.addEventHandler('hardware_id', 'base-selected', bioassays.setHardwareCallback);
			
			// Software
			Buttons.addClickHandler('software_id.select', bioassays.selectSoftware);
			Events.addEventHandler('software_id', 'base-selected', bioassays.setSoftwareCallback);
			
			// Parent derived bioassays
			Buttons.addClickHandler('btnAddParents', bioassays.addParents);
			
			// Parent physical bioassays
			Buttons.addClickHandler('btnAddPhysicalBioAssays', bioassays.addPhysicalBioAssays);
			
			// Parent type
			Events.addEventHandler('isRoot', 'click', bioassays.isRootOnChange);
			Events.addEventHandler('isChild', 'click', bioassays.isRootOnChange);
			var frm = document.forms['bioAssay'];
			if (frm.isRoot) 
			{
				bioassays.isRootOnChange();
			}
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'DERIVEDBIOASSAY', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnNewChildBioAssay', bioassays.newChildBioAssay, attributes);
			Buttons.addClickHandler('btnNewRawBioAssay', bioassays.newRawBioAssay, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunAnalysisPlugin', Buttons.runPlugin, attributes);
			
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			TabControl.addTabActivateListener('main.overview', Overview.loadOnce);
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'DERIVEDBIOASSAY'};
			var tableAttributes = {'table-id': 'bioassays'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnInheritAnnotations', Buttons.inheritAnnotations, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnNewMergedDerivedBioAssay', bioassays.newMergedDerivedBioAssay);
			Buttons.addClickHandler('btnNewItemList', bioassays.newItemListOnClick);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	bioassays.initElements = function(element, autoInit)
	{
		if (autoInit == 'new-child-bioassay')
		{
			Buttons.addClickHandler(element, bioassays.newChildBioAssay);
		}
		else if (autoInit == 'new-raw-bioassay')
		{
			Buttons.addClickHandler(element, bioassays.newRawBioAssay);
		}
	}
	
	bioassays.newMergedDerivedBioAssay = function()
	{
		if (!Table.checkIfSelected('bioassays'))
		{
			return;
		}
		Table.submitToPopup('bioassays', 'NewMergedDerivedBioAssay', 750, 500);
	}

	bioassays.newChildBioAssay = function(event)
	{
		var parentId = Data.int(event.currentTarget, 'item-id');
		Items.newItem('DERIVEDBIOASSAY', '&parent_id='+parentId);
	}

	bioassays.newRawBioAssay = function(event)
	{
		var parentId = Data.int(event.currentTarget, 'item-id');
		Items.newItem('RAWBIOASSAY', '&bioassay_id='+parentId);
	}
	
	// Create a new item list with the selected items as members
	bioassays.newItemListOnClick = function()
	{
		var controller = Items.getController('ITEMLIST');
		Table.submitToPopup('bioassays', 'CreateItemList', controller.width, controller.height);
	}

	
	bioassays.validateBioAssay = function()
	{
		var frm = document.forms['bioAssay'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		return true;
	}

	bioassays.save = function()
	{
		var frm = document.forms['bioAssay'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			DataFiles.writeFileActionsToForm(frm);
			if (Doc.element('physicalBioAssays')) 
			{
				Link.exportActions('physicalBioAssays');
			}
			if (Doc.element('parents')) 
			{
				Link.exportActions('parents');
			}
			frm.submit();
		}
	}
	
	bioassays.loadAnnotationsFrame = function()
	{
		Annotations.autoLoadEditFrame(bioassays.getProtocolId(), ItemSubtype.getSubtypeId('subtype_id'), bioassays.getParents());
	}

	bioassays.loadDataFilesFrame = function()
	{
		var subtypeId = ItemSubtype.getSubtypeId('subtype_id');
		DataFiles.autoLoadEditFrame(0, 0, subtypeId);
	}
	
	bioassays.getProtocolId = function()
	{
		var frm = document.forms['bioAssay'];
		var protocolId = 0;
		if (frm.protocol_id.length > 0 && !frm.protocol_id.disabled) 
		{
			protocolId = Math.abs(parseInt(frm.protocol_id.value));				
		}
		return protocolId;
	}
	
	bioassays.getParents = function()
	{
		var frm = document.forms['bioAssay'];
		
		var parents = [];
		if (frm.physicalBioAssays && (!frm.isRoot || frm.isRoot[0].checked))
		{
			var ids = Link.getIdsInList(frm.physicalBioAssays, 'PHYSICALBIOASSAY');
			if (ids.length > 0)
			{
				parents[parents.length] = 'PHYSICALBIOASSAY:'+ids.join(':');
			}
		}
		else if (frm.parents && (!frm.isRoot || frm.isRoot[1].checked))
		{
			var ids = Link.getIdsInList(frm.parents, 'DERIVEDBIOASSAY');
			if (ids.length > 0)
			{
				parents[parents.length] = 'DERIVEDBIOASSAY:'+ids.join(':');
			}
		}
		var extractId = Math.abs(parseInt(frm.extract_id.value));
		if (extractId > 0) parents[parents.length] = 'EXTRACT:'+extractId;
		var protocolId = Math.abs(parseInt(frm.protocol_id.value));
		if (protocolId > 0) parents[parents.length] = 'PROTOCOL:'+protocolId;
		var hardwareId = Math.abs(parseInt(frm.hardware_id.value));
		if (hardwareId > 0) parents[parents.length] = 'HARDWARE:'+hardwareId;
		var softwareId = Math.abs(parseInt(frm.software_id.value));
		if (softwareId > 0) parents[parents.length] = 'SOFTWARE:'+softwareId;
		return parents;
	}
	
	bioassays.subtypeOnChange = function()
	{
		var frm = document.forms['bioAssay'];
		var subtypeId = ItemSubtype.getSubtypeId('subtype_id');
		var recentInfo = ItemSubtype.getRelatedProjectDefaultAndRecentItems('DERIVEDBIOASSAY', subtypeId, ['PROTOCOL', 'HARDWARE', 'SOFTWARE']);
		ItemSubtype.updateSelectionList(frm.protocol_id, recentInfo.PROTOCOL['recent'], recentInfo.PROTOCOL['default']);
		ItemSubtype.updateSelectionList(frm.hardware_id, recentInfo.HARDWARE['recent'], recentInfo.HARDWARE['default']);
		ItemSubtype.updateSelectionList(frm.software_id, recentInfo.SOFTWARE['recent'], recentInfo.SOFTWARE['default']);
	}
	
	bioassays.selectProtocol = function()
	{
		var frm = document.forms['bioAssay'];
		var url = '&resetTemporary=1';
		url += ItemSubtype.createRelatedFilter('subtype_id', 'PROTOCOL');
		if (frm.protocol_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.protocol_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('PROTOCOL', 'protocol_id', 0, url);
	}
	
	bioassays.setProtocolCallback = function(event)
	{
		var frm = document.forms['bioAssay'];
		var list = frm.protocol_id;
		if (list.length < 2 || list[1].value == '0')
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}

	bioassays.selectHardware = function()
	{
		var frm = document.forms['bioAssay'];
		var url = '&resetTemporary=1';
		url += ItemSubtype.createRelatedFilter('subtype_id', 'HARDWARE');
		if (frm.hardware_id.length > 1)
		{
			var id = Math.abs(parseInt(frm.hardware_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('HARDWARE', 'hardware_id', 0, url);
	}
	
	bioassays.setHardwareCallback = function(event)
	{
		var frm = document.forms['bioAssay'];
		var list = frm.hardware_id;
		if (list.length < 2 || list[1].value == '0')
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}

	bioassays.selectSoftware = function()
	{
		var frm = document.forms['bioAssay'];
		var url = '&resetTemporary=1';
		url += ItemSubtype.createRelatedFilter('subtype_id', 'SOFTWARE');
		if (frm.software_id.length > 1)
		{
			var id = Math.abs(parseInt(frm.software_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('SOFTWARE', 'software_id', 0, url);
	}
	
	bioassays.setSoftwareCallback = function(event)
	{
		var frm = document.forms['bioAssay'];
		var list = frm.software_id;
		if (list.length < 2 || list[1].value == '0')
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}
	
	bioassays.isRootOnChange = function()
	{
		var frm = document.forms['bioAssay'];
		var isRoot = frm.isRoot[0].checked;
		Doc.showHide('physicalBioAssaySection', isRoot);
		Doc.showHide('parentBioAssaySection', !isRoot);
	}

	bioassays.addParents = function()
	{
		var frm = document.forms['bioAssay'];
		var current = Link.getIdsInList(frm.parents, 'DERIVEDBIOASSAY');
		var url = "&resetTemporary=1&exclude="+current.join(',');
		url += ItemSubtype.createRelatedFilter('subtype_id', 'DERIVEDBIOASSAY');
		Dialogs.selectItem('DERIVEDBIOASSAY', 'parents', 1, url);
	}
	
	bioassays.addPhysicalBioAssays = function()
	{
		var frm = document.forms['bioAssay'];
		var current = Link.getIdsInList(frm.physicalBioAssays, 'PHYSICALBIOASSAY');
		var url = "&resetTemporary=1&exclude="+current.join(',');
		url += ItemSubtype.createRelatedFilter('subtype_id', 'PHYSICALBIOASSAY');
		Dialogs.selectItem('PHYSICALBIOASSAY', 'physicalBioAssays', 1, url);
	}
	
	bioassays.selectExtract = function()
	{
		var frm = document.forms['bioAssay'];
		
		var relatedExtractIds = null;
		if (frm.physicalBioAssays && (!frm.isRoot || frm.isRoot[0].checked))
		{
			var ids = Link.getIdsInList(frm.physicalBioAssays, 'PHYSICALBIOASSAY');
			if (ids.length > 0)
			{
				var request = Ajax.getXmlHttpRequest();
				var url = '../physicalbioassays/ajax.jsp?ID='+App.getSessionId();
				url += '&cmd=GetAllExtractIds&bioassay_ids=' + ids.join(',');
				request.open("GET", url, false); 
				request.send(null);
				var response = JSON.parse(request.responseText);
				if (response.status != 'ok')
				{
					alert(response.message);
					return null;
				}
				relatedExtractIds = response.allExtractIds;
			}
		}
		else if (frm.parents && (!frm.isRoot || frm.isRoot[1].checked))
		{
			var ids = Link.getIdsInList(frm.parents, 'DERIVEDBIOASSAY');
			if (ids.length > 0)
			{
				var request = Ajax.getXmlHttpRequest();
				var url = 'ajax.jsp?ID='+App.getSessionId();
				url += '&cmd=GetAllExtractIds&bioassay_ids=' + ids.join(',');
				request.open("GET", url, false); 
				request.send(null);
				var response = JSON.parse(request.responseText);
				if (response.status != 'ok')
				{
					alert(response.message);
					return null;
				}
				relatedExtractIds = response.allExtractIds;
			}
		}

		var url = '&resetTemporary=1';
		if (relatedExtractIds) 
		{
			url += '&tmpfilter:INT:id='+relatedExtractIds.join('|');
		}
		url += ItemSubtype.createRelatedFilter('subtype_id', 'EXTRACT');
		if (frm.extract_id.length > 1)
		{
			var id = Math.abs(parseInt(frm.extract_id[1].value));				
			url += '&item_id='+id;
		}
		Dialogs.selectItem('EXTRACT', 'extract_id', 0, url);
	}
	
	bioassays.setExtractCallback = function(event)
	{
		var frm = document.forms['bioAssay'];
		var list = frm.extract_id;
		if (list.length < 2 || list[1].value == '0') // >
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}
	
	return bioassays;
}();

Doc.onLoad(BioAssays.initPage);
Doc.addElementInitializer(BioAssays.initElements);

